__all__ = [
    "Anchor",
    "Breadcrumb",
    "Button",
    "Checkbox",
    "DatePicker",
    "Form",
    "FormItem",
    "Input",
    "Message",
    "Modal",
    "Page",
    "PopoverConfirm",
    "Row",
    "Select",
    "Spin",
    "Switch",
    "Table",
    "TextArea",
]

from .components.anchor import Anchor
from .components.breadcrumb import Breadcrumb
from .components.button import Button
from .components.checkbox import Checkbox
from .components.date_picker import DatePicker
from .components.form import Form
from .components.form_item import FormItem
from .components.input import Input
from .components.message import Message
from .components.modal import Modal
from .components.page import Page
from .components.popover_confirm import PopoverConfirm
from .components.row import Row
from .components.select import Select
from .components.spin import Spin
from .components.switch import Switch
from .components.table import Table
from .components.text_area import TextArea



from selenium.webdriver.common.by import By
from transstellar.framework import Element


class Row(Element):
    XPATH_CURRENT = '//tr[contains(@class, "ant-table-row")]'

    column_titles = {}

    def set_column_titles(self, column_titles):
        self.column_titles = column_titles

    def get_cell_text(self, column_title: str):
        self.logger.debug(f"get cell text of column: {column_title}")

        cell = self.__get_cell_dom_element(column_title)

        return cell.text.strip()

    def find_dom_element_in_cell_by_xpath(self, column_title: str, xpath: str):
        self.logger.debug(
            f"get element in cell of column {column_title} by xpath: {xpath}"
        )
        cell = self.__get_cell_dom_element(column_title)

        dom_element = cell.find_element(By.XPATH, f".{xpath}")

        self.sleep(0.5)

        return dom_element

    def get_cells(self):
        cells = {}

        for column_title in self.column_titles:
            cells[column_title] = self.get_cell_text(column_title)

        return cells

    def __get_cell_dom_element(self, column_name: str):
        self.logger.debug(f"get cell of column {column_name}")

        column_index = self.column_titles[column_name]
        cells = self.find_dom_elements_by_tag_name("td")

        return cells[column_index]



__all__ = [
    "Badge",
    "Button",
    "Breadcrumb",
    "Checkbox",
    "Collapse",
    "ColorPicker",
    "DatePicker",
    "Form",
    "FormItem",
    "Input",
    "Menu",
    "Message",
    "Modal",
    "Page",
    "Popover",
    "PopoverConfirm",
    "RadioGroup",
    "Row",
    "Select",
    "Spin",
    "Switch",
    "Tab",
    "Table",
    "TextArea",
    "Upload",
]

from .badge import Badge
from .breadcrumb import Breadcrumb
from .button import Button
from .checkbox import Checkbox
from .collapse import Collapse
from .color_picker import ColorPicker
from .date_picker import DatePicker
from .form import Form
from .form_item import FormItem
from .input import Input
from .menu import Menu
from .message import Message
from .modal import Modal
from .page import Page
from .popover import Popover
from .popover_confirm import PopoverConfirm
from .radio_group import RadioGroup
from .row import Row
from .select import Select
from .spin import Spin
from .switch import Switch
from .tab import Tab
from .table import Table
from .text_area import TextArea
from .upload import Upload



import time

from transstellar.framework import BasePage

from .message import Message


class Page(BasePage):
    MESSAGE_CLASS = Message

    def get_ant_message(self) -> str:
        message_element: Message = self.find_global_element(self.MESSAGE_CLASS)

        return message_element.get_content()

    def wait_until_ant_message(self, message: str, timeout: int = 10) -> str:
        start_time = time.time()

        while True:
            current_time = time.time()

            if current_time - start_time > timeout:
                raise TimeoutError(f"could not find message: {message}")

            if self.get_ant_message() == message:
                return message

            time.sleep(1)



import re

from selenium.webdriver.common.keys import Keys
from transstellar.framework import Element

from .popover import Popover
from .select import Select


class ColorPicker(Element):
    XPATH_CURRENT = '//div[contains(@class, "ant-color-picker-trigger")]'
    XPATH_TRIGGER_COLOR = (
        '//div[contains(@class, "ant-color-picker-color-block-inner")]'
    )

    XPATH_PICKER = '//div[contains(@class, "ant-popover ant-color-picker")]'
    XPATH_COLOR_CODE = '//div[@class="ant-color-picker-input"]//input'

    POPOVER_CLASS = Popover
    SELECT_CLASS = Select

    CODE_TYPE_HEX = "HEX"
    CODE_TYPE_HSB = "HSB"
    CODE_TYPE_RGB = "RGB"

    def is_enabled(self):
        return "ant-color-picker-trigger-disabled" not in self.get_classes()

    def pick_color_by_hex(self, hex_code: str):
        self.logger.info(f"pick color by hex: {hex_code}")

        self.__update_color_by_code(self.CODE_TYPE_HEX, hex_code)

    def get_current_color(self):
        dom_element = self.find_dom_element_by_xpath(self.XPATH_TRIGGER_COLOR)
        style = dom_element.get_attribute("style")
        hex_code = self.__rgb_to_hex(style)

        return hex_code

    def __update_color_by_code(self, code_type: str, color_code: str):
        self.click()
        # Wait for popover to show
        self.sleep(4)

        self.picker_popover = self.find_global_element(self.POPOVER_CLASS)
        self.code_type_select = self.picker_popover.find_element(self.SELECT_CLASS)
        self.code_type_select.select(code_type)

        input_element = self.picker_popover.find_dom_element_by_xpath(
            self.XPATH_COLOR_CODE
        )

        input_element.send_keys(Keys.CONTROL + "a")
        input_element.send_keys(Keys.DELETE)
        input_element.send_keys(color_code)
        input_element.send_keys(Keys.ENTER)

        # Wait for popover to hide
        self.sleep(3)

        new_value = input_element.get_attribute("value")

        assert new_value.lower() == color_code.lower()

        return new_value

    def __rgb_to_hex(self, rgb_str: str):
        match = re.search(r"rgb\s*\(\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)\s*\)", rgb_str)
        if not match:
            raise ValueError("Invalid RGB format")

        r, g, b = map(int, match.groups())

        hex_color = f"{r:02x}{g:02x}{b:02x}"

        return hex_color.upper()



from transstellar.framework import Element


class Upload(Element):
    XPATH_CURRENT = '//div[contains(@class, "ant-upload ant-upload-select")]'

    def is_enabled(self):
        element = self.find_dom_element_by_xpath("//button")

        return element.get_attribute("disabled") is None



from transstellar.framework import Element


class Button(Element):
    XPATH_CURRENT = '//button[contains(@class, "ant-btn")]'

    def is_enabled(self):
        return self.get_current_dom_element().get_attribute("disabled") is None

    def click(self):
        self.dom_element.click()

    def get_text(self) -> str:
        return self.dom_element.text



from selenium.webdriver.common.keys import Keys
from transstellar.framework import Element


class Input(Element):
    XPATH_CURRENT = '//input[contains(@class, "ant-input")]'

    def is_enabled(self):
        return "ant-input-disabled" not in self.get_classes()

    def input(self, value: str):
        self.logger.info(f"input to value: {value}")

        dom_element = self.get_current_dom_element()
        dom_element.send_keys(Keys.CONTROL + "a")
        dom_element.send_keys(Keys.DELETE)
        dom_element.send_keys(value)

        new_value = self.get_value()

        assert new_value == value

    def append(self, value: str):
        self.logger.info(f"input to value: {value}")

        self.get_current_dom_element().send_keys(value)

    def get_value(self):
        return self.dom_element.get_attribute("value")



from transstellar.framework import Element

from .button import Button


class PopoverConfirm(Element):
    XPATH_CURRENT = '//div[contains(@class, "ant-popover ant-popconfirm")]'
    XPATH_POPOVER_TITLE = '//div[@class="ant-popover-message-title"]'
    BUTTON_CLASS = Button

    def get_popover_confirm_xpath(self, message):
        return f'{self.XPATH_CURRENT} and {self.XPATH_POPOVER_TITLE}//button[contains(span/text(), "{message}")]'

    def click(self, button_text):
        self.logger.info(f"click {button_text} in popover confirm")
        button: Button = self.find_element_by_label(self.BUTTON_CLASS, button_text)

        # NOTE: need time to be clickable
        self.sleep(0.5)

        button.click()



from transstellar.framework import Element


class Tab(Element):
    XPATH_CURRENT = (
        '//div[contains(@class, "ant-tabs")]//div[contains(@class, "ant-tabs-tab")]'
    )

    def click(self):
        self.get_current_dom_element().click()



from ...components import Upload as BaseUpload


class Upload(BaseUpload):
    XPATH_INNER = '//span[contains(@class, "ant-upload")]'

    def is_enabled(self):
        element = self.find_dom_element_by_xpath(self.XPATH_INNER)
        classes = element.get_attribute("class")

        return "ant-upload-disabled" not in classes



from transstellar.framework import Element

from .checkbox import Checkbox
from .form_item import FormItem
from .input import Input
from .select import Select
from .switch import Switch
from .text_area import TextArea


class Form(Element):
    XPATH_CURRENT = "//form"
    INPUT_CLASS = Input
    TEXT_AREA_CLASS = TextArea
    SELECT_CLASS = Select
    SWITCH_CLASS = Switch
    CHECKBOX_CLASS = Checkbox

    def input(self, form_item_label: str, value: str):
        input_element: Input = self.find_element_by_form_item_label(
            self.INPUT_CLASS, form_item_label
        )
        input_element.input(value)

    def text_area_input(self, form_item_label: str, value: str):
        text_area_element: TextArea = self.find_element_by_form_item_label(
            self.TEXT_AREA_CLASS,
            form_item_label,
        )
        text_area_element.input(value)

    def select(self, form_item_label: str, value: str):
        select_element: Select = self.find_element_by_form_item_label(
            self.SELECT_CLASS,
            form_item_label,
        )
        select_element.select(value)

    def switch(self, form_item_label: str, on: bool):
        switch_element: Switch = self.find_element_by_form_item_label(
            self.SWITCH_CLASS,
            form_item_label,
        )
        switch_element.switch(on)

    def check(self, form_item_label: str, on: bool):
        checkbox_element: Checkbox = self.find_element_by_form_item_label(
            self.CHECKBOX_CLASS,
            form_item_label,
        )
        checkbox_element.check(on)

    def direct_check(self, label: str, on: bool):
        checkbox_element: Checkbox = self.find_element_by_label(
            self.CHECKBOX_CLASS, label
        )
        checkbox_element.check(on)

    def find_element_by_form_item_label(self, element_class, form_item_label: str):
        form_item: FormItem = self.find_element_by_label(FormItem, form_item_label)
        element: element_class = form_item.find_form_control(element_class)

        return element

    def is_form_item_present(self, form_item_label: str):
        try:
            self.find_element_by_label(FormItem, form_item_label)

            return True
        except Exception:
            return False



from transstellar.framework import Element


class Menu(Element):
    XPATH_CURRENT = '//ul[contains(@class, "ant-menu ")]'
    XPATH_CURRENT_ITEM_TITLE = '//li[contains(@class, "ant-menu-item-selected")]'
    XPATH_TARGET_ITEM_TEMPLATE = (
        '//li[contains(@class, "ant-menu-item") and contains(.//text(), "{title}")]'
    )

    def select(self, text):
        self.logger.info(f"selecting text: {text}")

        self.get_current_dom_element().click()

        select_item = self.find_global_dom_element_by_xpath(
            self.XPATH_TARGET_ITEM_TEMPLATE.format(title=text)
        )
        select_item.click()

        # NOTE: need to find a better way to assert account has been totally changed
        self.sleep(0.5)

    def get_current_item_title(self):
        dom_element = self.find_dom_element_by_xpath(self.XPATH_CURRENT_ITEM_TITLE)

        return dom_element.text



from ...components import Select as BaseSelect


class Select(BaseSelect):
    XPATH_TARGET_ITEM_TEMPLATE = (
        '//div[contains(@class, "ant-select-item") and contains(@title, "{title}")]'
    )



from transstellar.framework import Element


class Checkbox(Element):
    XPATH_CURRENT = '//label[contains(@class, "ant-checkbox-wrapper")]'
    XPATH_CHECKBOX = '//span[contains(@class, "ant-checkbox")]'

    def get_checkbox_dom_element(self):
        dom_element = self.find_dom_element_by_xpath(self.XPATH_CHECKBOX)

        return dom_element

    def is_enabled(self):
        dom_element = self.find_dom_element_by_xpath(self.XPATH_CHECKBOX)

        return "ant-checkbox-disabled" not in dom_element.get_attribute("class")

    def check(self, on: bool):
        self.logger.info(f"check on: {on}")

        ant_checkbox = self.get_checkbox_dom_element()
        class_names = ant_checkbox.get_attribute("class")
        current_checked = "ant-checkbox-checked" in class_names.split()

        should_click = (not current_checked and on) or (current_checked and not on)
        self.logger.debug(
            f"current_checked: {current_checked}, should_click: {should_click}, on: {on}"
        )

        if should_click:
            ant_checkbox.click()

        current_checked = ant_checkbox.get_attribute("aria-checked")

        updated_class_names = ant_checkbox.get_attribute("class")
        current_checked = "ant-checkbox-checked" in updated_class_names.split()

        assert current_checked == on



from transstellar.framework import Element


class RadioGroup(Element):
    XPATH_CURRENT = '//div[contains(@class, "ant-radio-group")]'

    def is_enabled(self):
        radios = self.find_elements(Radio)

        return next((radio for radio in radios if radio.is_enabled()), False)

    def select(self, value):
        self.logger.info(f"Select radio: {value}")

        radio = self.find_element_by_label(Radio, value)
        radio.select()

    def get_current_item_text(self):
        dom_element = self.find_dom_element_by_xpath(Radio.XPATH_CHECKED)

        return dom_element.text


class Radio(Element):
    XPATH_CURRENT = '//label[contains(@class, "ant-radio-button-wrapper")]'
    XPATH_CHECKED = '//label[contains(@class, "ant-radio-button-wrapper-checked")]'

    def is_enabled(self):
        return "ant-radio-button-wrapper-disabled" not in self.get_classes()

    def select(self):
        self.click()
        self.sleep(2)

        assert "ant-radio-button-wrapper-checked" in self.get_classes()



from transstellar.framework import Element


class Modal(Element):
    XPATH_CURRENT = '//div[contains(@class, "ant-modal-content")]'

    XPATH_CLOSE_BUTTON = '//button[contains(@class, "ant-modal-close")]'

    def close(self):
        close_button = self.find_dom_element_by_xpath(self.XPATH_CLOSE_BUTTON)
        close_button.click()

        self.wait_for_dom_element_to_disappear_by_xpath(self.XPATH_CLOSE_BUTTON)

    def get_title(self):
        title_block = self.find_dom_element_by_xpath(
            '//div[contains(@class, "ant-modal-title")]'
        )

        return title_block.text



from urllib.parse import urlparse

from transstellar.framework import Element


class FormItem(Element):
    XPATH_CURRENT = '//div[contains(@class, "ant-form-item")]'

    def find_form_control(self, element_class):
        self.logger.info(
            f"finding form control element with label: {self.label}, element: {element_class.__name__}"
        )

        return self.find_element(element_class)



from transstellar.framework import Element


class Popover(Element):
    XPATH_CURRENT = '//div[contains(@class, "ant-popover ")]'
    XPATH_POPOVER_TITLE = '//div[@class="ant-popover-title"]'

    def is_opened(self):
        return "ant-popover-hidden" not in self.get_classes()

    def get_title(self):
        title_element = self.find_dom_element_by_xpath(self.XPATH_POPOVER_TITLE)

        return title_element.text



from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from transstellar.framework import Element


class DatePicker(Element):
    XPATH_CURRENT = '//div[contains(@class, "ant-picker-input")]/ancestor::div[contains(@class, "ant-picker")]'

    XPATH_INPUT = '//div[contains(@class, "ant-picker-input")]/input'

    def pick_date(self, target_date, is_datetime=False):
        self.logger.info(f"pick date: {target_date}")

        dom_element = self.find_dom_element_by_xpath(self.XPATH_INPUT)

        self.__update_date(dom_element, target_date, is_datetime)

    def pick_date_range(self, from_date, to_date, is_datetime=False):
        self.logger.info(f"pick date range, from: {from_date}, to: {to_date}")

        inputs = self.find_dom_elements_by_tag_name("input")
        from_date_input = inputs[0]
        to_date_input = inputs[1]

        self.__update_date(from_date_input, from_date, is_datetime)
        self.__update_date(to_date_input, to_date, is_datetime)

    def get_basic_date_value(self):
        dom_element = self.find_dom_element_by_xpath(self.XPATH_INPUT)

        return dom_element.get_attribute("value")

    def get_date_range_values(self):
        inputs = self.find_dom_elements_by_tag_name("input")

        return [inputs[0].get_attribute("value"), inputs[1].get_attribute("value")]

    def __update_date(self, input_element, date, is_datetime=False):
        date_format = "%Y-%m-%d"

        if is_datetime:
            date_format = "%Y-%m-%d %H:%M:%S"

        formatted_date = date.strftime(date_format)

        input_element.click()

        # Wait for calendar to show
        self.sleep(3)

        input_element.send_keys(Keys.CONTROL + "a")
        input_element.send_keys(Keys.DELETE)
        input_element.send_keys(formatted_date)
        input_element.send_keys(Keys.ENTER)

        # Wait for calendar to hide
        self.sleep(3)

        new_value = input_element.get_attribute("value")

        assert new_value == formatted_date

        return new_value



from transstellar.framework import Element


class Message(Element):
    XPATH_CURRENT = '//div[contains(@class, "ant-message ")]/div/div'

    def get_content(self):
        return self.get_current_dom_element().get_attribute("textContent")



from transstellar.framework import Element


class Switch(Element):
    XPATH_CURRENT = '//button[contains(@class, "ant-switch")]'

    def is_enabled(self):
        return "ant-switch-disabled" not in self.get_classes()

    def switch(self, on: bool):
        self.logger.info(f"switch on: {on}")

        ant_switch = self.get_current_dom_element()
        current_checked = ant_switch.get_attribute("aria-checked")

        should_click = (current_checked == "true" and not on) or (
            current_checked == "false" and on
        )

        if should_click:
            ant_switch.click()
            self.sleep(0.5)

        current_checked = ant_switch.get_attribute("aria-checked")

        if on:
            assert current_checked == "true"
        else:
            assert current_checked == "false"



from transstellar.framework import Element


class Collapse(Element):
    XPATH_CURRENT = '//div[contains(@class, "ant-collapse ")]'

    def get_items(self):
        collapse_elements = self.find_elements(CollapseItem)

        return collapse_elements

    def find_item(self, header: str):
        collapse_element = self.find_element_by_label(CollapseItem, header)

        return collapse_element


class CollapseItem(Element):
    XPATH_CURRENT = '//div[contains(@class, "ant-collapse-item")]'
    XPATH_HEADER = '//div[@class="ant-collapse-header"]'
    XPATH_CONTENT = '//div[contains(@class, "ant-collapse-content")]'

    def click(self):
        self.get_header().click()
        self.sleep(1)

    def is_expanded(self):
        header = self.get_header()
        expanded_text = header.get_attribute("aria-expanded")

        return expanded_text == "true"

    def get_header(self):
        dom_element = self.find_dom_element_by_xpath(self.XPATH_HEADER)

        return dom_element

    def get_header_text(self):
        dom_element = self.get_header()

        return dom_element.text

    def get_content(self):
        dom_element = self.find_dom_element_by_xpath(self.XPATH_CONTENT)

        return dom_element

    def get_content_text(self):
        dom_element = self.get_content()

        return dom_element.text



from ...components import Form as BaseForm
from .checkbox import Checkbox
from .form_item import FormItem
from .input import Input
from .select import Select
from .switch import Switch
from .text_area import TextArea


class Form(BaseForm):
    INPUT_CLASS = Input
    TEXT_AREA_CLASS = TextArea
    SELECT_CLASS = Select
    SWITCH_CLASS = Switch
    CHECKBOX_CLASS = Checkbox

    pass



from transstellar.framework import Element


class Anchor(Element):
    XPATH_CURRENT = "//a"

    def click(self):
        self.dom_element.click()

    def get_text(self) -> str:
        return self.dom_element.text



from selenium.webdriver.common.keys import Keys
from transstellar.framework import Element


class Select(Element):
    XPATH_CURRENT = '//div[contains(@class, "ant-select ")]'
    XPATH_SELECT_SEARCH = (
        '//input[contains(@class, "ant-select-selection-search-input")]'
    )
    XPATH_CURRENT_ITEM_TITLE = '//span[contains(@class, "ant-select-selection-item")]'
    XPATH_TARGET_ITEM_TEMPLATE = (
        '//div[contains(@class, "ant-select-item") and contains(@title, "{title}")]'
    )

    def is_enabled(self):
        return "ant-select-disabled" not in self.get_classes()

    def select(self, text):
        self.logger.info(f"selecting text: {text}")

        self.get_current_dom_element().click()

        select_item = self.find_global_dom_element_by_xpath(
            self.XPATH_TARGET_ITEM_TEMPLATE.format(title=text)
        )
        select_item.click()

        # NOTE: need to find a better way to assert account has been totally changed
        self.sleep(0.5)

    def select_by_search(self, text):
        self.logger.info(f"selecting text: {text}")

        self.get_current_dom_element().click()

        search_input = self.find_dom_element_by_xpath(self.XPATH_SELECT_SEARCH)
        search_input.send_keys(text)
        search_input.send_keys(Keys.RETURN)

        # NOTE: need to find a better way to assert account has been totally changed
        self.sleep(0.5)

    def get_current_item_title(self):
        dom_element = self.find_dom_element_by_xpath(self.XPATH_CURRENT_ITEM_TITLE)

        return dom_element.get_attribute("title")



from typing import List, Union

from selenium.webdriver.common.by import By
from transstellar.framework import Element

from .row import Row


class Table(Element):
    XPATH_CURRENT = '//div[contains(@class, "ant-table-wrapper")]//div[contains(@class, "ant-table")]'
    SELECTOR_TABLE_HEADER = "thead.ant-table-thead th.ant-table-cell"
    SELECTOR_ROW = "tbody.ant-table-tbody tr.ant-table-row"
    ROW_CLASS = Row

    rows: List[Row]
    column_titles = {}

    def init_after_dom_element_is_set(self):
        header_columns = self.dom_element.find_elements(
            By.CSS_SELECTOR, self.SELECTOR_TABLE_HEADER
        )

        for index, column in enumerate(header_columns):
            column_name = column.get_attribute("innerHTML").strip()

            # Sometimes there is a column used as scrollbar
            if column_name == "":
                continue

            self.column_titles[column_name] = index

        if not self.is_empty():
            self.rows = self.find_elements(self.ROW_CLASS)

            for row in self.rows:
                row.set_column_titles(self.column_titles)

    def is_empty(self):
        classes = self.get_current_dom_element().get_attribute("class")

        return "ant-table-empty" in classes

    def find_row(self, column_title: str, column_text: str) -> Union[None, Row]:
        self.logger.info(f"finding row with column {column_title}: {column_text}")

        column_index = self.column_titles[column_title]

        if column_index is not None:
            for row in self.rows:
                if row.get_cell_text(column_title) == str(column_text):
                    self.logger.debug(
                        f"found row with column {column_title} as {column_text}"
                    )

                    return row

        raise LookupError(
            f"could not find row with column {column_title} as {column_text}"
        )

    def get_row(self, index: int):
        self.logger.info(f"finding row by index: {index}")

        return self.rows[index]
